@@ -18,10 +18,6 @@
 //! for this task, such as `sled`:
 //! <https://github.com/spacejam/sled>
 //!
-//! This library is once primarily for my own personal use. I simply decided to open-source it just in case
-//! anyone finds this useful. There are no file headers or data versioning support in the created file,
-//! but the format of the data should remain stable for the forseeable future.
-//!
 //! # Example
 //!
 //! ```ignore
@@ -70,6 +66,9 @@ use std::{
 };
 use thiserror::Error;
 
+const FILE_HEADER: u32 = 67297350;
+const FILE_VERSION: u32 = 2;
+
 /// This is the main error type of this crate.
 #[derive(Error, Debug)]
 pub enum DatabaseError<T> {
@@ -88,6 +87,17 @@ pub enum DatabaseError<T> {
     /// Serialization/deserialization error for an object.
     #[error(transparent)]
     SerdeError(#[from] bincode::Error),
+    /// Invalid file header. You might not be reading a valid `crio` file.
+    #[error("invalid file header")]
+    WrongFileHeader,
+    /// Wrong file version. Use another version of this library to read the file correctly.
+    ///
+    /// # File versions:
+    ///
+    /// 1: 0.2 versions and below
+    /// 2: 0.3 versions and above
+    #[error("wrong file version: expected {}, found {0}", FILE_VERSION)]
+    WrongFileVersion(u32),
 }
 
 #[derive(Error, Debug)]
@@ -222,6 +232,7 @@ where
 }
 
 fn binary_to_vec<T: DeserializeOwned>(mut raw_data: &[u8]) -> Result<Vec<T>, DatabaseError<T>> {
+    validate_collection(&mut raw_data)?;
     let mut is_corrupted = None;
     let mut result = Vec::new();
     loop {
@@ -262,8 +273,26 @@ fn process_document<R: Read>(f: &mut R) -> Result<Vec<u8>, InnerDatabaseError> {
     Ok(data)
 }
 
+fn validate_collection<R: Read, T>(f: &mut R) -> Result<(), DatabaseError<T>> {
+    let saved_file_header = f
+        .read_u32::<LittleEndian>()
+        .map_err(|_| DatabaseError::WrongFileHeader)?;
+    if saved_file_header != FILE_HEADER {
+        return Err(DatabaseError::WrongFileHeader);
+    }
+    let saved_file_version = f
+        .read_u32::<LittleEndian>()
+        .map_err(|_| DatabaseError::WrongFileVersion(1))?;
+    if saved_file_version != FILE_VERSION {
+        return Err(DatabaseError::WrongFileVersion(saved_file_version));
+    }
+    Ok(())
+}
+
 fn vec_to_binary<T: Serialize>(data: &[T]) -> Result<Vec<u8>, DatabaseError<T>> {
     let mut buf = Vec::new();
+    buf.write_u32::<LittleEndian>(FILE_HEADER)?;
+    buf.write_u32::<LittleEndian>(FILE_VERSION)?;
     for document in data {
         let raw_data = bincode::serialize(&document)?;
         let data_len = raw_data.len();
